(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.egraph = {}));
}(this, (function (exports) { 'use strict';

    let wasm;

    const heap = new Array(128).fill(undefined);

    heap.push(undefined, null, true, false);

    function getObject(idx) { return heap[idx]; }

    let heap_next = heap.length;

    function dropObject(idx) {
        if (idx < 132) return;
        heap[idx] = heap_next;
        heap_next = idx;
    }

    function takeObject(idx) {
        const ret = getObject(idx);
        dropObject(idx);
        return ret;
    }

    function addHeapObject(obj) {
        if (heap_next === heap.length) heap.push(heap.length + 1);
        const idx = heap_next;
        heap_next = heap[idx];

        heap[idx] = obj;
        return idx;
    }

    const cachedTextDecoder = new TextDecoder('utf-8', { ignoreBOM: true, fatal: true });

    cachedTextDecoder.decode();

    let cachedUint8Memory0 = null;

    function getUint8Memory0() {
        if (cachedUint8Memory0 === null || cachedUint8Memory0.byteLength === 0) {
            cachedUint8Memory0 = new Uint8Array(wasm.memory.buffer);
        }
        return cachedUint8Memory0;
    }

    function getStringFromWasm0(ptr, len) {
        return cachedTextDecoder.decode(getUint8Memory0().subarray(ptr, ptr + len));
    }

    function isLikeNone(x) {
        return x === undefined || x === null;
    }

    let cachedFloat64Memory0 = null;

    function getFloat64Memory0() {
        if (cachedFloat64Memory0 === null || cachedFloat64Memory0.byteLength === 0) {
            cachedFloat64Memory0 = new Float64Array(wasm.memory.buffer);
        }
        return cachedFloat64Memory0;
    }

    let cachedInt32Memory0 = null;

    function getInt32Memory0() {
        if (cachedInt32Memory0 === null || cachedInt32Memory0.byteLength === 0) {
            cachedInt32Memory0 = new Int32Array(wasm.memory.buffer);
        }
        return cachedInt32Memory0;
    }

    let WASM_VECTOR_LEN = 0;

    const cachedTextEncoder = new TextEncoder('utf-8');

    const encodeString = (typeof cachedTextEncoder.encodeInto === 'function'
        ? function (arg, view) {
        return cachedTextEncoder.encodeInto(arg, view);
    }
        : function (arg, view) {
        const buf = cachedTextEncoder.encode(arg);
        view.set(buf);
        return {
            read: arg.length,
            written: buf.length
        };
    });

    function passStringToWasm0(arg, malloc, realloc) {

        if (realloc === undefined) {
            const buf = cachedTextEncoder.encode(arg);
            const ptr = malloc(buf.length);
            getUint8Memory0().subarray(ptr, ptr + buf.length).set(buf);
            WASM_VECTOR_LEN = buf.length;
            return ptr;
        }

        let len = arg.length;
        let ptr = malloc(len);

        const mem = getUint8Memory0();

        let offset = 0;

        for (; offset < len; offset++) {
            const code = arg.charCodeAt(offset);
            if (code > 0x7F) break;
            mem[ptr + offset] = code;
        }

        if (offset !== len) {
            if (offset !== 0) {
                arg = arg.slice(offset);
            }
            ptr = realloc(ptr, len, len = offset + arg.length * 3);
            const view = getUint8Memory0().subarray(ptr + offset, ptr + len);
            const ret = encodeString(arg, view);

            offset += ret.written;
        }

        WASM_VECTOR_LEN = offset;
        return ptr;
    }

    let cachedBigInt64Memory0 = null;

    function getBigInt64Memory0() {
        if (cachedBigInt64Memory0 === null || cachedBigInt64Memory0.byteLength === 0) {
            cachedBigInt64Memory0 = new BigInt64Array(wasm.memory.buffer);
        }
        return cachedBigInt64Memory0;
    }

    function debugString(val) {
        // primitive types
        const type = typeof val;
        if (type == 'number' || type == 'boolean' || val == null) {
            return  `${val}`;
        }
        if (type == 'string') {
            return `"${val}"`;
        }
        if (type == 'symbol') {
            const description = val.description;
            if (description == null) {
                return 'Symbol';
            } else {
                return `Symbol(${description})`;
            }
        }
        if (type == 'function') {
            const name = val.name;
            if (typeof name == 'string' && name.length > 0) {
                return `Function(${name})`;
            } else {
                return 'Function';
            }
        }
        // objects
        if (Array.isArray(val)) {
            const length = val.length;
            let debug = '[';
            if (length > 0) {
                debug += debugString(val[0]);
            }
            for(let i = 1; i < length; i++) {
                debug += ', ' + debugString(val[i]);
            }
            debug += ']';
            return debug;
        }
        // Test for built-in
        const builtInMatches = /\[object ([^\]]+)\]/.exec(toString.call(val));
        let className;
        if (builtInMatches.length > 1) {
            className = builtInMatches[1];
        } else {
            // Failed to match the standard '[object ClassName]'
            return toString.call(val);
        }
        if (className == 'Object') {
            // we're a user defined class or Object
            // JSON.stringify avoids problems with cycles, and is generally much
            // easier than looping through ownProperties of `val`.
            try {
                return 'Object(' + JSON.stringify(val) + ')';
            } catch (_) {
                return 'Object';
            }
        }
        // errors
        if (val instanceof Error) {
            return `${val.name}: ${val.message}\n${val.stack}`;
        }
        // TODO we could test for more things here, like `Set`s and `Map`s.
        return className;
    }

    function _assertClass(instance, klass) {
        if (!(instance instanceof klass)) {
            throw new Error(`expected instance of ${klass.name}`);
        }
        return instance.ptr;
    }
    /**
    * @param {Graph} graph
    * @param {any} coordinates
    * @returns {any}
    */
    function fdeb(graph, coordinates) {
        _assertClass(graph, Graph);
        const ret = wasm.fdeb(graph.ptr, addHeapObject(coordinates));
        return takeObject(ret);
    }

    let cachedFloat32Memory0 = null;

    function getFloat32Memory0() {
        if (cachedFloat32Memory0 === null || cachedFloat32Memory0.byteLength === 0) {
            cachedFloat32Memory0 = new Float32Array(wasm.memory.buffer);
        }
        return cachedFloat32Memory0;
    }

    let stack_pointer = 128;

    function addBorrowedObject(obj) {
        if (stack_pointer == 1) throw new Error('out of js stack');
        heap[--stack_pointer] = obj;
        return stack_pointer;
    }
    /**
    * @param {Graph} graph
    * @param {Function} groups
    * @param {Function} shrink_node
    * @param {Function} shrink_edge
    * @returns {any}
    */
    function coarsen(graph, groups, shrink_node, shrink_edge) {
        try {
            const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
            _assertClass(graph, Graph);
            wasm.coarsen(retptr, graph.ptr, addBorrowedObject(groups), addBorrowedObject(shrink_node), addBorrowedObject(shrink_edge));
            var r0 = getInt32Memory0()[retptr / 4 + 0];
            var r1 = getInt32Memory0()[retptr / 4 + 1];
            var r2 = getInt32Memory0()[retptr / 4 + 2];
            if (r2) {
                throw takeObject(r1);
            }
            return takeObject(r0);
        } finally {
            wasm.__wbindgen_add_to_stack_pointer(16);
            heap[stack_pointer++] = undefined;
            heap[stack_pointer++] = undefined;
            heap[stack_pointer++] = undefined;
        }
    }

    /**
    * @param {Graph} graph
    * @param {Drawing} drawing
    * @returns {number}
    */
    function stress(graph, drawing) {
        _assertClass(graph, Graph);
        _assertClass(drawing, Drawing);
        const ret = wasm.stress(graph.ptr, drawing.ptr);
        return ret;
    }

    /**
    * @param {Graph} graph
    * @param {Drawing} drawing
    * @returns {number}
    */
    function crossingNumber(graph, drawing) {
        _assertClass(graph, Graph);
        _assertClass(drawing, Drawing);
        const ret = wasm.crossingNumber(graph.ptr, drawing.ptr);
        return ret;
    }

    /**
    * @param {Graph} graph
    * @param {Drawing} drawing
    * @returns {number}
    */
    function neighborhoodPreservation(graph, drawing) {
        _assertClass(graph, Graph);
        _assertClass(drawing, Drawing);
        const ret = wasm.neighborhoodPreservation(graph.ptr, drawing.ptr);
        return ret;
    }

    /**
    * @param {Graph} graph
    * @param {number} min_size
    * @param {number} step_iteration
    * @param {Function} _shrink_node
    * @param {Function} _shrink_edge
    * @param {Function} _link_distance_accessor
    * @returns {Coordinates}
    */
    function fm3(graph, min_size, step_iteration, _shrink_node, _shrink_edge, _link_distance_accessor) {
        _assertClass(graph, Graph);
        const ret = wasm.fm3(graph.ptr, min_size, step_iteration, addHeapObject(_shrink_node), addHeapObject(_shrink_edge), addHeapObject(_link_distance_accessor));
        return Coordinates.__wrap(ret);
    }

    function handleError(f, args) {
        try {
            return f.apply(this, args);
        } catch (e) {
            wasm.__wbindgen_exn_store(addHeapObject(e));
        }
    }

    function getArrayU8FromWasm0(ptr, len) {
        return getUint8Memory0().subarray(ptr / 1, ptr / 1 + len);
    }
    /**
    */
    class ClassicalMds {

        static __wrap(ptr) {
            const obj = Object.create(ClassicalMds.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_classicalmds_free(ptr);
        }
        /**
        */
        constructor() {
            const ret = wasm.classicalmds_new();
            return ClassicalMds.__wrap(ret);
        }
        /**
        * @param {Graph} graph
        * @param {Function} length
        * @returns {Drawing}
        */
        run(graph, length) {
            try {
                _assertClass(graph, Graph);
                const ret = wasm.classicalmds_run(this.ptr, graph.ptr, addBorrowedObject(length));
                return Drawing.__wrap(ret);
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
    }
    /**
    */
    class CollideForce {

        static __wrap(ptr) {
            const obj = Object.create(CollideForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_collideforce_free(ptr);
        }
        /**
        * @param {Graph} graph
        * @param {Function} f
        * @param {any} options
        */
        constructor(graph, f, options) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                _assertClass(graph, Graph);
                wasm.collideforce_new(retptr, graph.ptr, addBorrowedObject(f), addBorrowedObject(options));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return CollideForce.__wrap(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
                heap[stack_pointer++] = undefined;
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Coordinates} context
        * @param {number} alpha
        */
        apply(context, alpha) {
            _assertClass(context, Coordinates);
            wasm.collideforce_apply(this.ptr, context.ptr, alpha);
        }
    }
    /**
    */
    class Coordinates {

        static __wrap(ptr) {
            const obj = Object.create(Coordinates.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_coordinates_free(ptr);
        }
        /**
        * @param {number} u
        * @returns {number | undefined}
        */
        x(u) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.coordinates_x(retptr, this.ptr, u);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getFloat32Memory0()[retptr / 4 + 1];
                return r0 === 0 ? undefined : r1;
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} u
        * @returns {number | undefined}
        */
        y(u) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.coordinates_y(retptr, this.ptr, u);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getFloat32Memory0()[retptr / 4 + 1];
                return r0 === 0 ? undefined : r1;
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} u
        * @param {number} x
        */
        setX(u, x) {
            wasm.coordinates_setX(this.ptr, u, x);
        }
        /**
        * @param {number} u
        * @param {number} y
        */
        setY(u, y) {
            wasm.coordinates_setY(this.ptr, u, y);
        }
        /**
        * @param {number} u
        * @returns {number | undefined}
        */
        vx(u) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.coordinates_vx(retptr, this.ptr, u);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getFloat32Memory0()[retptr / 4 + 1];
                return r0 === 0 ? undefined : r1;
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} u
        * @returns {number | undefined}
        */
        vy(u) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.coordinates_vy(retptr, this.ptr, u);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getFloat32Memory0()[retptr / 4 + 1];
                return r0 === 0 ? undefined : r1;
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @returns {number}
        */
        len() {
            const ret = wasm.coordinates_len(this.ptr);
            return ret >>> 0;
        }
        /**
        * @returns {any}
        */
        toJSON() {
            const ret = wasm.coordinates_toJSON(this.ptr);
            return takeObject(ret);
        }
        /**
        */
        centralize() {
            wasm.coordinates_centralize(this.ptr);
        }
        /**
        * @param {number} velocity_decay
        */
        updatePosition(velocity_decay) {
            wasm.coordinates_updatePosition(this.ptr, velocity_decay);
        }
        /**
        * @param {number} x0
        * @param {number} y0
        * @param {number} x1
        * @param {number} y1
        */
        clampRegion(x0, y0, x1, y1) {
            wasm.coordinates_clampRegion(this.ptr, x0, y0, x1, y1);
        }
        /**
        * @param {Graph} graph
        * @returns {Coordinates}
        */
        static initialPlacement(graph) {
            _assertClass(graph, Graph);
            const ret = wasm.coordinates_initialPlacement(graph.ptr);
            return Coordinates.__wrap(ret);
        }
    }
    /**
    */
    class DiGraph {

        static __wrap(ptr) {
            const obj = Object.create(DiGraph.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_digraph_free(ptr);
        }
        /**
        */
        constructor() {
            const ret = wasm.digraph_new();
            return DiGraph.__wrap(ret);
        }
        /**
        * @returns {number}
        */
        nodeCount() {
            const ret = wasm.digraph_nodeCount(this.ptr);
            return ret >>> 0;
        }
        /**
        * @returns {number}
        */
        edgeCount() {
            const ret = wasm.digraph_edgeCount(this.ptr);
            return ret >>> 0;
        }
        /**
        * @param {any} value
        * @returns {number}
        */
        addNode(value) {
            const ret = wasm.digraph_addNode(this.ptr, addHeapObject(value));
            return ret >>> 0;
        }
        /**
        * @param {number} a
        * @returns {any}
        */
        nodeWeight(a) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.digraph_nodeWeight(retptr, this.ptr, a);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} a
        * @param {number} b
        * @param {any} value
        * @returns {number}
        */
        addEdge(a, b, value) {
            const ret = wasm.digraph_addEdge(this.ptr, a, b, addHeapObject(value));
            return ret >>> 0;
        }
        /**
        * @param {number} e
        * @returns {any}
        */
        edgeWeight(e) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.digraph_edgeWeight(retptr, this.ptr, e);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} e
        * @returns {Array<any>}
        */
        edgeEndpoints(e) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.digraph_edgeEndpoints(retptr, this.ptr, e);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} a
        * @returns {any}
        */
        removeNode(a) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.digraph_removeNode(retptr, this.ptr, a);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} e
        * @returns {any}
        */
        removeEdge(e) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.digraph_removeEdge(retptr, this.ptr, e);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} a
        * @returns {Array<any>}
        */
        neighbors(a) {
            const ret = wasm.digraph_neighbors(this.ptr, a);
            return takeObject(ret);
        }
        /**
        * @param {number} a
        * @param {number} dir
        * @returns {Array<any>}
        */
        neighborsDirected(a, dir) {
            const ret = wasm.digraph_neighborsDirected(this.ptr, a, dir);
            return takeObject(ret);
        }
        /**
        * @param {number} a
        * @returns {Array<any>}
        */
        neighborsUndirected(a) {
            const ret = wasm.digraph_neighborsUndirected(this.ptr, a);
            return takeObject(ret);
        }
        /**
        * @param {number} a
        * @returns {Array<any>}
        */
        edges(a) {
            const ret = wasm.digraph_edges(this.ptr, a);
            return takeObject(ret);
        }
        /**
        * @param {number} a
        * @param {number} b
        * @returns {boolean}
        */
        containsEdge(a, b) {
            const ret = wasm.digraph_containsEdge(this.ptr, a, b);
            return ret !== 0;
        }
        /**
        * @param {number} a
        * @param {number} b
        * @returns {number}
        */
        findEdge(a, b) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.digraph_findEdge(retptr, this.ptr, a, b);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return r0 >>> 0;
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} dir
        * @returns {Array<any>}
        */
        externals(dir) {
            const ret = wasm.digraph_externals(this.ptr, dir);
            return takeObject(ret);
        }
        /**
        * @returns {Array<any>}
        */
        nodeIndices() {
            const ret = wasm.digraph_nodeIndices(this.ptr);
            return takeObject(ret);
        }
        /**
        * @returns {Array<any>}
        */
        edgeIndices() {
            const ret = wasm.digraph_edgeIndices(this.ptr);
            return takeObject(ret);
        }
        /**
        * @param {Function} node_map
        * @param {Function} edge_map
        * @returns {DiGraph}
        */
        map(node_map, edge_map) {
            try {
                const ret = wasm.digraph_map(this.ptr, addBorrowedObject(node_map), addBorrowedObject(edge_map));
                return DiGraph.__wrap(ret);
            } finally {
                heap[stack_pointer++] = undefined;
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Function} node_map
        * @param {Function} edge_map
        * @returns {DiGraph}
        */
        filterMap(node_map, edge_map) {
            try {
                const ret = wasm.digraph_filterMap(this.ptr, addBorrowedObject(node_map), addBorrowedObject(edge_map));
                return DiGraph.__wrap(ret);
            } finally {
                heap[stack_pointer++] = undefined;
                heap[stack_pointer++] = undefined;
            }
        }
    }
    /**
    */
    class Drawing {

        static __wrap(ptr) {
            const obj = Object.create(Drawing.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_drawing_free(ptr);
        }
        /**
        * @param {number} u
        * @returns {number | undefined}
        */
        x(u) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.drawing_x(retptr, this.ptr, u);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getFloat32Memory0()[retptr / 4 + 1];
                return r0 === 0 ? undefined : r1;
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} u
        * @returns {number | undefined}
        */
        y(u) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.drawing_y(retptr, this.ptr, u);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getFloat32Memory0()[retptr / 4 + 1];
                return r0 === 0 ? undefined : r1;
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} u
        * @param {number} x
        */
        setX(u, x) {
            wasm.drawing_setX(this.ptr, u, x);
        }
        /**
        * @param {number} u
        * @param {number} y
        */
        setY(u, y) {
            wasm.drawing_setY(this.ptr, u, y);
        }
        /**
        * @returns {number}
        */
        len() {
            const ret = wasm.drawing_len(this.ptr);
            return ret >>> 0;
        }
        /**
        */
        centralize() {
            wasm.drawing_centralize(this.ptr);
        }
        /**
        * @param {number} x0
        * @param {number} y0
        * @param {number} x1
        * @param {number} y1
        */
        clampRegion(x0, y0, x1, y1) {
            wasm.drawing_clampRegion(this.ptr, x0, y0, x1, y1);
        }
        /**
        * @param {Graph} graph
        * @returns {Drawing}
        */
        static initialPlacement(graph) {
            _assertClass(graph, Graph);
            const ret = wasm.drawing_initialPlacement(graph.ptr);
            return Drawing.__wrap(ret);
        }
    }
    /**
    */
    class ForceAtlas2 {

        static __wrap(ptr) {
            const obj = Object.create(ForceAtlas2.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_forceatlas2_free(ptr);
        }
        /**
        * @param {Graph} graph
        */
        constructor(graph) {
            _assertClass(graph, Graph);
            const ret = wasm.forceatlas2_new(graph.ptr);
            return ForceAtlas2.__wrap(ret);
        }
        /**
        * @param {Drawing} drawing
        * @param {number} alpha
        */
        apply(drawing, alpha) {
            _assertClass(drawing, Drawing);
            wasm.forceatlas2_apply(this.ptr, drawing.ptr, alpha);
        }
        /**
        * @param {number} u
        * @param {Drawing} drawing
        * @param {number} alpha
        */
        applyToNode(u, drawing, alpha) {
            _assertClass(drawing, Drawing);
            wasm.forceatlas2_applyToNode(this.ptr, u, drawing.ptr, alpha);
        }
    }
    /**
    */
    class FruchtermanReingoldForce {

        static __wrap(ptr) {
            const obj = Object.create(FruchtermanReingoldForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_fruchtermanreingoldforce_free(ptr);
        }
        /**
        * @param {Graph} graph
        * @param {number} k
        * @param {number} min_distance
        */
        constructor(graph, k, min_distance) {
            _assertClass(graph, Graph);
            const ret = wasm.fruchtermanreingoldforce_new(graph.ptr, k, min_distance);
            return FruchtermanReingoldForce.__wrap(ret);
        }
        /**
        * @param {Coordinates} coordinates
        * @param {number} alpha
        */
        apply(coordinates, alpha) {
            _assertClass(coordinates, Coordinates);
            wasm.fruchtermanreingoldforce_apply(this.ptr, coordinates.ptr, alpha);
        }
        /**
        * @param {number} u
        * @param {Coordinates} coordinates
        * @param {number} alpha
        */
        applyToNode(u, coordinates, alpha) {
            _assertClass(coordinates, Coordinates);
            wasm.fruchtermanreingoldforce_applyToNode(this.ptr, u, coordinates.ptr, alpha);
        }
    }
    /**
    */
    class FullSgd {

        static __wrap(ptr) {
            const obj = Object.create(FullSgd.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_fullsgd_free(ptr);
        }
        /**
        * @param {Graph} graph
        * @param {Function} length
        */
        constructor(graph, length) {
            try {
                _assertClass(graph, Graph);
                const ret = wasm.fullsgd_new(graph.ptr, addBorrowedObject(length));
                return FullSgd.__wrap(ret);
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Rng} rng
        */
        shuffle(rng) {
            _assertClass(rng, Rng);
            wasm.fullsgd_shuffle(this.ptr, rng.ptr);
        }
        /**
        * @param {Drawing} drawing
        * @param {number} eta
        */
        apply(drawing, eta) {
            _assertClass(drawing, Drawing);
            wasm.fullsgd_apply(this.ptr, drawing.ptr, eta);
        }
        /**
        * @param {number} t_max
        * @param {number} epsilon
        * @returns {SgdScheduler}
        */
        scheduler(t_max, epsilon) {
            const ret = wasm.fullsgd_scheduler(this.ptr, t_max, epsilon);
            return SgdScheduler.__wrap(ret);
        }
    }
    /**
    */
    class Graph {

        static __wrap(ptr) {
            const obj = Object.create(Graph.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_graph_free(ptr);
        }
        /**
        */
        constructor() {
            const ret = wasm.digraph_new();
            return Graph.__wrap(ret);
        }
        /**
        * @returns {number}
        */
        nodeCount() {
            const ret = wasm.digraph_nodeCount(this.ptr);
            return ret >>> 0;
        }
        /**
        * @returns {number}
        */
        edgeCount() {
            const ret = wasm.digraph_edgeCount(this.ptr);
            return ret >>> 0;
        }
        /**
        * @param {any} value
        * @returns {number}
        */
        addNode(value) {
            const ret = wasm.digraph_addNode(this.ptr, addHeapObject(value));
            return ret >>> 0;
        }
        /**
        * @param {number} a
        * @returns {any}
        */
        nodeWeight(a) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.digraph_nodeWeight(retptr, this.ptr, a);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} a
        * @param {number} b
        * @param {any} value
        * @returns {number}
        */
        addEdge(a, b, value) {
            const ret = wasm.graph_addEdge(this.ptr, a, b, addHeapObject(value));
            return ret >>> 0;
        }
        /**
        * @param {number} e
        * @returns {any}
        */
        edgeWeight(e) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.digraph_edgeWeight(retptr, this.ptr, e);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} e
        * @returns {Array<any>}
        */
        edgeEndpoints(e) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.digraph_edgeEndpoints(retptr, this.ptr, e);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} a
        * @returns {any}
        */
        removeNode(a) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.graph_removeNode(retptr, this.ptr, a);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} e
        * @returns {any}
        */
        removeEdge(e) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.graph_removeEdge(retptr, this.ptr, e);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} a
        * @returns {Array<any>}
        */
        neighbors(a) {
            const ret = wasm.graph_neighbors(this.ptr, a);
            return takeObject(ret);
        }
        /**
        * @param {number} a
        * @param {number} dir
        * @returns {Array<any>}
        */
        neighborsDirected(a, dir) {
            const ret = wasm.graph_neighborsDirected(this.ptr, a, dir);
            return takeObject(ret);
        }
        /**
        * @param {number} a
        * @returns {Array<any>}
        */
        neighborsUndirected(a) {
            const ret = wasm.digraph_neighborsUndirected(this.ptr, a);
            return takeObject(ret);
        }
        /**
        * @param {number} a
        * @returns {Array<any>}
        */
        edges(a) {
            const ret = wasm.graph_edges(this.ptr, a);
            return takeObject(ret);
        }
        /**
        * @param {number} a
        * @param {number} b
        * @returns {boolean}
        */
        containsEdge(a, b) {
            const ret = wasm.graph_containsEdge(this.ptr, a, b);
            return ret !== 0;
        }
        /**
        * @param {number} a
        * @param {number} b
        * @returns {number}
        */
        findEdge(a, b) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.graph_findEdge(retptr, this.ptr, a, b);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return r0 >>> 0;
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} dir
        * @returns {Array<any>}
        */
        externals(dir) {
            const ret = wasm.graph_externals(this.ptr, dir);
            return takeObject(ret);
        }
        /**
        * @returns {Array<any>}
        */
        nodeIndices() {
            const ret = wasm.digraph_nodeIndices(this.ptr);
            return takeObject(ret);
        }
        /**
        * @returns {Array<any>}
        */
        edgeIndices() {
            const ret = wasm.digraph_edgeIndices(this.ptr);
            return takeObject(ret);
        }
        /**
        * @param {Function} node_map
        * @param {Function} edge_map
        * @returns {Graph}
        */
        map(node_map, edge_map) {
            try {
                const ret = wasm.graph_map(this.ptr, addBorrowedObject(node_map), addBorrowedObject(edge_map));
                return Graph.__wrap(ret);
            } finally {
                heap[stack_pointer++] = undefined;
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Function} node_map
        * @param {Function} edge_map
        * @returns {Graph}
        */
        filterMap(node_map, edge_map) {
            try {
                const ret = wasm.graph_filterMap(this.ptr, addBorrowedObject(node_map), addBorrowedObject(edge_map));
                return Graph.__wrap(ret);
            } finally {
                heap[stack_pointer++] = undefined;
                heap[stack_pointer++] = undefined;
            }
        }
    }
    /**
    */
    class GroupLinkForce {

        static __wrap(ptr) {
            const obj = Object.create(GroupLinkForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_grouplinkforce_free(ptr);
        }
        /**
        * @param {Graph} graph
        * @param {Function} group_accessor
        * @param {Function} inter_link_accessor
        * @param {Function} intra_link_accessor
        */
        constructor(graph, group_accessor, inter_link_accessor, intra_link_accessor) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                _assertClass(graph, Graph);
                wasm.grouplinkforce_new(retptr, graph.ptr, addBorrowedObject(group_accessor), addBorrowedObject(inter_link_accessor), addBorrowedObject(intra_link_accessor));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return GroupLinkForce.__wrap(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
                heap[stack_pointer++] = undefined;
                heap[stack_pointer++] = undefined;
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Coordinates} coordinates
        * @param {number} alpha
        */
        apply(coordinates, alpha) {
            _assertClass(coordinates, Coordinates);
            wasm.grouplinkforce_apply(this.ptr, coordinates.ptr, alpha);
        }
    }
    /**
    */
    class GroupManyBodyForce {

        static __wrap(ptr) {
            const obj = Object.create(GroupManyBodyForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_groupmanybodyforce_free(ptr);
        }
        /**
        * @param {Graph} graph
        * @param {Function} node_accessor
        */
        constructor(graph, node_accessor) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                _assertClass(graph, Graph);
                wasm.groupmanybodyforce_new(retptr, graph.ptr, addBorrowedObject(node_accessor));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return GroupManyBodyForce.__wrap(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Coordinates} coordinates
        * @param {number} alpha
        */
        apply(coordinates, alpha) {
            _assertClass(coordinates, Coordinates);
            wasm.groupmanybodyforce_apply(this.ptr, coordinates.ptr, alpha);
        }
    }
    /**
    */
    class GroupPositionForce {

        static __wrap(ptr) {
            const obj = Object.create(GroupPositionForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_grouppositionforce_free(ptr);
        }
        /**
        * @param {Graph} graph
        * @param {Function} node_accessor
        * @param {Function} group_accessor
        */
        constructor(graph, node_accessor, group_accessor) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                _assertClass(graph, Graph);
                wasm.grouppositionforce_new(retptr, graph.ptr, addBorrowedObject(node_accessor), addBorrowedObject(group_accessor));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return GroupPositionForce.__wrap(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
                heap[stack_pointer++] = undefined;
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Coordinates} coordinates
        * @param {number} alpha
        */
        apply(coordinates, alpha) {
            _assertClass(coordinates, Coordinates);
            wasm.grouppositionforce_apply(this.ptr, coordinates.ptr, alpha);
        }
    }
    /**
    */
    class HyperbolicSpace {

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_hyperbolicspace_free(ptr);
        }
        /**
        * @param {any} a
        * @param {any} b
        * @returns {any}
        */
        static toTangentSpace(a, b) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.hyperbolicspace_toTangentSpace(retptr, addHeapObject(a), addHeapObject(b));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {any} a
        * @param {any} b
        * @returns {any}
        */
        static fromTangentSpace(a, b) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.hyperbolicspace_fromTangentSpace(retptr, addHeapObject(a), addHeapObject(b));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} i
        * @param {Coordinates} riemann_space
        * @param {Coordinates} tangent_space
        */
        static mapToTangentSpace(i, riemann_space, tangent_space) {
            _assertClass(riemann_space, Coordinates);
            _assertClass(tangent_space, Coordinates);
            wasm.hyperbolicspace_mapToTangentSpace(i, riemann_space.ptr, tangent_space.ptr);
        }
        /**
        * @param {number} i
        * @param {Coordinates} riemann_space
        * @param {Coordinates} tangent_space
        * @param {number} velocity_decay
        */
        static updatePosition(i, riemann_space, tangent_space, velocity_decay) {
            _assertClass(riemann_space, Coordinates);
            _assertClass(tangent_space, Coordinates);
            wasm.hyperbolicspace_updatePosition(i, riemann_space.ptr, tangent_space.ptr, velocity_decay);
        }
    }
    /**
    */
    class KamadaKawai {

        static __wrap(ptr) {
            const obj = Object.create(KamadaKawai.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_kamadakawai_free(ptr);
        }
        /**
        * @param {Graph} graph
        * @param {Function} f
        */
        constructor(graph, f) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                _assertClass(graph, Graph);
                wasm.kamadakawai_new(retptr, graph.ptr, addBorrowedObject(f));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return KamadaKawai.__wrap(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Drawing} drawing
        * @returns {number | undefined}
        */
        selectNode(drawing) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                _assertClass(drawing, Drawing);
                wasm.kamadakawai_selectNode(retptr, this.ptr, drawing.ptr);
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                return r0 === 0 ? undefined : r1 >>> 0;
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} m
        * @param {Drawing} drawing
        */
        applyToNode(m, drawing) {
            _assertClass(drawing, Drawing);
            wasm.kamadakawai_applyToNode(this.ptr, m, drawing.ptr);
        }
        /**
        * @param {Drawing} drawing
        */
        run(drawing) {
            _assertClass(drawing, Drawing);
            wasm.kamadakawai_run(this.ptr, drawing.ptr);
        }
        /**
        * @returns {number}
        */
        get eps() {
            const ret = wasm.kamadakawai_eps(this.ptr);
            return ret;
        }
        /**
        * @param {number} value
        */
        set eps(value) {
            wasm.kamadakawai_set_eps(this.ptr, value);
        }
    }
    /**
    */
    class LinkForce {

        static __wrap(ptr) {
            const obj = Object.create(LinkForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_linkforce_free(ptr);
        }
        /**
        * @param {Graph} graph
        * @param {Function} f
        */
        constructor(graph, f) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                _assertClass(graph, Graph);
                wasm.linkforce_new(retptr, graph.ptr, addBorrowedObject(f));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return LinkForce.__wrap(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Coordinates} coordinates
        * @param {number} alpha
        */
        apply(coordinates, alpha) {
            _assertClass(coordinates, Coordinates);
            wasm.linkforce_apply(this.ptr, coordinates.ptr, alpha);
        }
        /**
        * @param {number} u
        * @param {Coordinates} coordinates
        * @param {number} alpha
        */
        applyToNode(u, coordinates, alpha) {
            _assertClass(coordinates, Coordinates);
            wasm.linkforce_applyToNode(this.ptr, u, coordinates.ptr, alpha);
        }
    }
    /**
    */
    class ManyBodyForce {

        static __wrap(ptr) {
            const obj = Object.create(ManyBodyForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_manybodyforce_free(ptr);
        }
        /**
        * @param {Graph} graph
        * @param {Function} f
        */
        constructor(graph, f) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                _assertClass(graph, Graph);
                wasm.manybodyforce_new(retptr, graph.ptr, addBorrowedObject(f));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return ManyBodyForce.__wrap(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Coordinates} coordinates
        * @param {number} alpha
        */
        apply(coordinates, alpha) {
            _assertClass(coordinates, Coordinates);
            wasm.manybodyforce_apply(this.ptr, coordinates.ptr, alpha);
        }
        /**
        * @param {number} u
        * @param {Coordinates} coordinates
        * @param {number} alpha
        */
        applyToNode(u, coordinates, alpha) {
            _assertClass(coordinates, Coordinates);
            wasm.manybodyforce_applyToNode(this.ptr, u, coordinates.ptr, alpha);
        }
    }
    /**
    */
    class PivotMds {

        static __wrap(ptr) {
            const obj = Object.create(PivotMds.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_pivotmds_free(ptr);
        }
        /**
        */
        constructor() {
            const ret = wasm.pivotmds_new();
            return PivotMds.__wrap(ret);
        }
        /**
        * @param {Graph} graph
        * @param {Function} length
        * @param {Array<any>} sources
        * @returns {Drawing}
        */
        run(graph, length, sources) {
            try {
                _assertClass(graph, Graph);
                const ret = wasm.pivotmds_run(this.ptr, graph.ptr, addBorrowedObject(length), addBorrowedObject(sources));
                return Drawing.__wrap(ret);
            } finally {
                heap[stack_pointer++] = undefined;
                heap[stack_pointer++] = undefined;
            }
        }
    }
    /**
    */
    class PositionForce {

        static __wrap(ptr) {
            const obj = Object.create(PositionForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_positionforce_free(ptr);
        }
        /**
        * @param {Graph} graph
        * @param {Function} f
        */
        constructor(graph, f) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                _assertClass(graph, Graph);
                wasm.positionforce_new(retptr, graph.ptr, addBorrowedObject(f));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return PositionForce.__wrap(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Coordinates} coordinates
        * @param {number} alpha
        */
        apply(coordinates, alpha) {
            _assertClass(coordinates, Coordinates);
            wasm.positionforce_apply(this.ptr, coordinates.ptr, alpha);
        }
        /**
        * @param {number} u
        * @param {Coordinates} coordinates
        * @param {number} alpha
        */
        applyToNode(u, coordinates, alpha) {
            _assertClass(coordinates, Coordinates);
            wasm.positionforce_applyToNode(this.ptr, u, coordinates.ptr, alpha);
        }
    }
    /**
    */
    class RadialForce {

        static __wrap(ptr) {
            const obj = Object.create(RadialForce.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_radialforce_free(ptr);
        }
        /**
        * @param {Graph} graph
        * @param {Function} f
        */
        constructor(graph, f) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                _assertClass(graph, Graph);
                wasm.radialforce_new(retptr, graph.ptr, addHeapObject(f));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return RadialForce.__wrap(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {Coordinates} coordinates
        * @param {number} alpha
        */
        apply(coordinates, alpha) {
            _assertClass(coordinates, Coordinates);
            wasm.radialforce_apply(this.ptr, coordinates.ptr, alpha);
        }
        /**
        * @param {number} u
        * @param {Coordinates} coordinates
        * @param {number} alpha
        */
        applyToNode(u, coordinates, alpha) {
            _assertClass(coordinates, Coordinates);
            wasm.radialforce_applyToNode(this.ptr, u, coordinates.ptr, alpha);
        }
    }
    /**
    */
    class Rng {

        static __wrap(ptr) {
            const obj = Object.create(Rng.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_rng_free(ptr);
        }
        /**
        */
        constructor() {
            const ret = wasm.rng_new();
            return Rng.__wrap(ret);
        }
        /**
        * @param {bigint} seed
        * @returns {Rng}
        */
        static seedFrom(seed) {
            const ret = wasm.rng_seedFrom(seed);
            return Rng.__wrap(ret);
        }
    }
    /**
    */
    class SgdScheduler {

        static __wrap(ptr) {
            const obj = Object.create(SgdScheduler.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_sgdscheduler_free(ptr);
        }
        /**
        * @param {Function} f
        */
        run(f) {
            try {
                wasm.sgdscheduler_run(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Function} f
        */
        step(f) {
            try {
                wasm.sgdscheduler_step(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @returns {boolean}
        */
        isFinished() {
            const ret = wasm.sgdscheduler_isFinished(this.ptr);
            return ret !== 0;
        }
    }
    /**
    */
    class Simulation {

        static __wrap(ptr) {
            const obj = Object.create(Simulation.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_simulation_free(ptr);
        }
        /**
        */
        constructor() {
            const ret = wasm.simulation_new();
            return Simulation.__wrap(ret);
        }
        /**
        * @param {Function} f
        */
        run(f) {
            try {
                wasm.simulation_run(this.ptr, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {number} n
        * @param {Function} f
        */
        runStep(n, f) {
            try {
                wasm.simulation_runStep(this.ptr, n, addBorrowedObject(f));
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @returns {boolean}
        */
        isFinished() {
            const ret = wasm.simulation_isFinished(this.ptr);
            return ret !== 0;
        }
        /**
        * @param {number} alpha_start
        */
        reset(alpha_start) {
            wasm.simulation_reset(this.ptr, alpha_start);
        }
        /**
        * @returns {number}
        */
        get alphaStart() {
            const ret = wasm.simulation_alpha(this.ptr);
            return ret;
        }
        /**
        * @param {number} value
        */
        set alphaStart(value) {
            wasm.simulation_set_alpha(this.ptr, value);
        }
        /**
        * @returns {number}
        */
        get alphaMin() {
            const ret = wasm.simulation_alpha_min(this.ptr);
            return ret;
        }
        /**
        * @param {number} value
        */
        set alphaMin(value) {
            wasm.simulation_set_alpha_min(this.ptr, value);
        }
        /**
        * @returns {number}
        */
        get alphaTarget() {
            const ret = wasm.simulation_alpha_target(this.ptr);
            return ret;
        }
        /**
        * @param {number} value
        */
        set alphaTarget(value) {
            wasm.simulation_set_alpha_target(this.ptr, value);
        }
        /**
        * @returns {number}
        */
        get iterations() {
            const ret = wasm.simulation_iterations(this.ptr);
            return ret >>> 0;
        }
        /**
        * @param {number} value
        */
        set iterations(value) {
            wasm.simulation_set_iterations(this.ptr, value);
        }
    }
    /**
    */
    class SparseSgd {

        static __wrap(ptr) {
            const obj = Object.create(SparseSgd.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_sparsesgd_free(ptr);
        }
        /**
        * @param {Graph} graph
        * @param {Function} length
        * @param {number} h
        * @param {Rng} rng
        */
        constructor(graph, length, h, rng) {
            try {
                _assertClass(graph, Graph);
                _assertClass(rng, Rng);
                const ret = wasm.sparsesgd_new(graph.ptr, addBorrowedObject(length), h, rng.ptr);
                return SparseSgd.__wrap(ret);
            } finally {
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Rng} rng
        */
        shuffle(rng) {
            _assertClass(rng, Rng);
            wasm.sparsesgd_shuffle(this.ptr, rng.ptr);
        }
        /**
        * @param {Drawing} drawing
        * @param {number} eta
        */
        apply(drawing, eta) {
            _assertClass(drawing, Drawing);
            wasm.sparsesgd_apply(this.ptr, drawing.ptr, eta);
        }
        /**
        * @param {number} t_max
        * @param {number} epsilon
        * @returns {SgdScheduler}
        */
        scheduler(t_max, epsilon) {
            const ret = wasm.sparsesgd_scheduler(this.ptr, t_max, epsilon);
            return SgdScheduler.__wrap(ret);
        }
    }
    /**
    */
    class SphericalSpace {

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_sphericalspace_free(ptr);
        }
        /**
        * @param {any} a
        * @param {any} b
        * @returns {any}
        */
        static toTangentSpace(a, b) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.sphericalspace_toTangentSpace(retptr, addHeapObject(a), addHeapObject(b));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {any} a
        * @param {any} b
        * @returns {any}
        */
        static fromTangentSpace(a, b) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                wasm.sphericalspace_fromTangentSpace(retptr, addHeapObject(a), addHeapObject(b));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return takeObject(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
            }
        }
        /**
        * @param {number} i
        * @param {Coordinates} riemann_space
        * @param {Coordinates} tangent_space
        */
        static mapToTangentSpace(i, riemann_space, tangent_space) {
            _assertClass(riemann_space, Coordinates);
            _assertClass(tangent_space, Coordinates);
            wasm.sphericalspace_mapToTangentSpace(i, riemann_space.ptr, tangent_space.ptr);
        }
        /**
        * @param {number} i
        * @param {Coordinates} riemann_space
        * @param {Coordinates} tangent_space
        * @param {number} velocity_decay
        */
        static updatePosition(i, riemann_space, tangent_space, velocity_decay) {
            _assertClass(riemann_space, Coordinates);
            _assertClass(tangent_space, Coordinates);
            wasm.sphericalspace_updatePosition(i, riemann_space.ptr, tangent_space.ptr, velocity_decay);
        }
    }
    /**
    */
    class StressMajorization {

        static __wrap(ptr) {
            const obj = Object.create(StressMajorization.prototype);
            obj.ptr = ptr;

            return obj;
        }

        __destroy_into_raw() {
            const ptr = this.ptr;
            this.ptr = 0;

            return ptr;
        }

        free() {
            const ptr = this.__destroy_into_raw();
            wasm.__wbg_stressmajorization_free(ptr);
        }
        /**
        * @param {Graph} graph
        * @param {Drawing} drawing
        * @param {Function} f
        */
        constructor(graph, drawing, f) {
            try {
                const retptr = wasm.__wbindgen_add_to_stack_pointer(-16);
                _assertClass(graph, Graph);
                _assertClass(drawing, Drawing);
                wasm.stressmajorization_new(retptr, graph.ptr, drawing.ptr, addBorrowedObject(f));
                var r0 = getInt32Memory0()[retptr / 4 + 0];
                var r1 = getInt32Memory0()[retptr / 4 + 1];
                var r2 = getInt32Memory0()[retptr / 4 + 2];
                if (r2) {
                    throw takeObject(r1);
                }
                return StressMajorization.__wrap(r0);
            } finally {
                wasm.__wbindgen_add_to_stack_pointer(16);
                heap[stack_pointer++] = undefined;
            }
        }
        /**
        * @param {Drawing} drawing
        * @returns {number}
        */
        apply(drawing) {
            _assertClass(drawing, Drawing);
            const ret = wasm.stressmajorization_apply(this.ptr, drawing.ptr);
            return ret;
        }
        /**
        * @param {Drawing} drawing
        */
        run(drawing) {
            _assertClass(drawing, Drawing);
            wasm.stressmajorization_run(this.ptr, drawing.ptr);
        }
    }

    async function load(module, imports) {
        if (typeof Response === 'function' && module instanceof Response) {
            if (typeof WebAssembly.instantiateStreaming === 'function') {
                try {
                    return await WebAssembly.instantiateStreaming(module, imports);

                } catch (e) {
                    if (module.headers.get('Content-Type') != 'application/wasm') {
                        console.warn("`WebAssembly.instantiateStreaming` failed because your server does not serve wasm with `application/wasm` MIME type. Falling back to `WebAssembly.instantiate` which is slower. Original error:\n", e);

                    } else {
                        throw e;
                    }
                }
            }

            const bytes = await module.arrayBuffer();
            return await WebAssembly.instantiate(bytes, imports);

        } else {
            const instance = await WebAssembly.instantiate(module, imports);

            if (instance instanceof WebAssembly.Instance) {
                return { instance, module };

            } else {
                return instance;
            }
        }
    }

    function getImports() {
        const imports = {};
        imports.wbg = {};
        imports.wbg.__wbindgen_object_drop_ref = function(arg0) {
            takeObject(arg0);
        };
        imports.wbg.__wbindgen_number_new = function(arg0) {
            const ret = arg0;
            return addHeapObject(ret);
        };
        imports.wbg.__wbindgen_string_new = function(arg0, arg1) {
            const ret = getStringFromWasm0(arg0, arg1);
            return addHeapObject(ret);
        };
        imports.wbg.__wbindgen_number_get = function(arg0, arg1) {
            const obj = getObject(arg1);
            const ret = typeof(obj) === 'number' ? obj : undefined;
            getFloat64Memory0()[arg0 / 8 + 1] = isLikeNone(ret) ? 0 : ret;
            getInt32Memory0()[arg0 / 4 + 0] = !isLikeNone(ret);
        };
        imports.wbg.__wbindgen_is_null = function(arg0) {
            const ret = getObject(arg0) === null;
            return ret;
        };
        imports.wbg.__wbindgen_is_bigint = function(arg0) {
            const ret = typeof(getObject(arg0)) === 'bigint';
            return ret;
        };
        imports.wbg.__wbindgen_bigint_from_u64 = function(arg0) {
            const ret = BigInt.asUintN(64, arg0);
            return addHeapObject(ret);
        };
        imports.wbg.__wbindgen_jsval_eq = function(arg0, arg1) {
            const ret = getObject(arg0) === getObject(arg1);
            return ret;
        };
        imports.wbg.__wbindgen_is_undefined = function(arg0) {
            const ret = getObject(arg0) === undefined;
            return ret;
        };
        imports.wbg.__wbindgen_object_clone_ref = function(arg0) {
            const ret = getObject(arg0);
            return addHeapObject(ret);
        };
        imports.wbg.__wbindgen_is_string = function(arg0) {
            const ret = typeof(getObject(arg0)) === 'string';
            return ret;
        };
        imports.wbg.__wbg_graph_new = function(arg0) {
            const ret = Graph.__wrap(arg0);
            return addHeapObject(ret);
        };
        imports.wbg.__wbindgen_error_new = function(arg0, arg1) {
            const ret = new Error(getStringFromWasm0(arg0, arg1));
            return addHeapObject(ret);
        };
        imports.wbg.__wbindgen_is_object = function(arg0) {
            const val = getObject(arg0);
            const ret = typeof(val) === 'object' && val !== null;
            return ret;
        };
        imports.wbg.__wbindgen_jsval_loose_eq = function(arg0, arg1) {
            const ret = getObject(arg0) == getObject(arg1);
            return ret;
        };
        imports.wbg.__wbindgen_boolean_get = function(arg0) {
            const v = getObject(arg0);
            const ret = typeof(v) === 'boolean' ? (v ? 1 : 0) : 2;
            return ret;
        };
        imports.wbg.__wbindgen_string_get = function(arg0, arg1) {
            const obj = getObject(arg1);
            const ret = typeof(obj) === 'string' ? obj : undefined;
            var ptr0 = isLikeNone(ret) ? 0 : passStringToWasm0(ret, wasm.__wbindgen_malloc, wasm.__wbindgen_realloc);
            var len0 = WASM_VECTOR_LEN;
            getInt32Memory0()[arg0 / 4 + 1] = len0;
            getInt32Memory0()[arg0 / 4 + 0] = ptr0;
        };
        imports.wbg.__wbg_String_91fba7ded13ba54c = function(arg0, arg1) {
            const ret = String(getObject(arg1));
            const ptr0 = passStringToWasm0(ret, wasm.__wbindgen_malloc, wasm.__wbindgen_realloc);
            const len0 = WASM_VECTOR_LEN;
            getInt32Memory0()[arg0 / 4 + 1] = len0;
            getInt32Memory0()[arg0 / 4 + 0] = ptr0;
        };
        imports.wbg.__wbg_set_20cbc34131e76824 = function(arg0, arg1, arg2) {
            getObject(arg0)[takeObject(arg1)] = takeObject(arg2);
        };
        imports.wbg.__wbg_randomFillSync_6894564c2c334c42 = function() { return handleError(function (arg0, arg1, arg2) {
            getObject(arg0).randomFillSync(getArrayU8FromWasm0(arg1, arg2));
        }, arguments) };
        imports.wbg.__wbg_getRandomValues_805f1c3d65988a5a = function() { return handleError(function (arg0, arg1) {
            getObject(arg0).getRandomValues(getObject(arg1));
        }, arguments) };
        imports.wbg.__wbg_crypto_e1d53a1d73fb10b8 = function(arg0) {
            const ret = getObject(arg0).crypto;
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_process_038c26bf42b093f8 = function(arg0) {
            const ret = getObject(arg0).process;
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_versions_ab37218d2f0b24a8 = function(arg0) {
            const ret = getObject(arg0).versions;
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_node_080f4b19d15bc1fe = function(arg0) {
            const ret = getObject(arg0).node;
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_msCrypto_6e7d3e1f92610cbb = function(arg0) {
            const ret = getObject(arg0).msCrypto;
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_require_78a3dcfbdba9cbce = function() { return handleError(function () {
            const ret = module.require;
            return addHeapObject(ret);
        }, arguments) };
        imports.wbg.__wbindgen_is_function = function(arg0) {
            const ret = typeof(getObject(arg0)) === 'function';
            return ret;
        };
        imports.wbg.__wbg_get_27fe3dac1c4d0224 = function(arg0, arg1) {
            const ret = getObject(arg0)[arg1 >>> 0];
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_length_e498fbc24f9c1d4f = function(arg0) {
            const ret = getObject(arg0).length;
            return ret;
        };
        imports.wbg.__wbg_new_b525de17f44a8943 = function() {
            const ret = new Array();
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_newnoargs_2b8b6bd7753c76ba = function(arg0, arg1) {
            const ret = new Function(getStringFromWasm0(arg0, arg1));
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_new_f841cc6f2098f4b5 = function() {
            const ret = new Map();
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_next_b7d530c04fd8b217 = function(arg0) {
            const ret = getObject(arg0).next;
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_next_88560ec06a094dea = function() { return handleError(function (arg0) {
            const ret = getObject(arg0).next();
            return addHeapObject(ret);
        }, arguments) };
        imports.wbg.__wbg_done_1ebec03bbd919843 = function(arg0) {
            const ret = getObject(arg0).done;
            return ret;
        };
        imports.wbg.__wbg_value_6ac8da5cc5b3efda = function(arg0) {
            const ret = getObject(arg0).value;
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_iterator_55f114446221aa5a = function() {
            const ret = Symbol.iterator;
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_get_baf4855f9a986186 = function() { return handleError(function (arg0, arg1) {
            const ret = Reflect.get(getObject(arg0), getObject(arg1));
            return addHeapObject(ret);
        }, arguments) };
        imports.wbg.__wbg_call_95d1ea488d03e4e8 = function() { return handleError(function (arg0, arg1) {
            const ret = getObject(arg0).call(getObject(arg1));
            return addHeapObject(ret);
        }, arguments) };
        imports.wbg.__wbg_new_f9876326328f45ed = function() {
            const ret = new Object();
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_self_e7c1f827057f6584 = function() { return handleError(function () {
            const ret = self.self;
            return addHeapObject(ret);
        }, arguments) };
        imports.wbg.__wbg_window_a09ec664e14b1b81 = function() { return handleError(function () {
            const ret = window.window;
            return addHeapObject(ret);
        }, arguments) };
        imports.wbg.__wbg_globalThis_87cbb8506fecf3a9 = function() { return handleError(function () {
            const ret = globalThis.globalThis;
            return addHeapObject(ret);
        }, arguments) };
        imports.wbg.__wbg_global_c85a9259e621f3db = function() { return handleError(function () {
            const ret = global.global;
            return addHeapObject(ret);
        }, arguments) };
        imports.wbg.__wbg_set_17224bc548dd1d7b = function(arg0, arg1, arg2) {
            getObject(arg0)[arg1 >>> 0] = takeObject(arg2);
        };
        imports.wbg.__wbg_isArray_39d28997bf6b96b4 = function(arg0) {
            const ret = Array.isArray(getObject(arg0));
            return ret;
        };
        imports.wbg.__wbg_push_49c286f04dd3bf59 = function(arg0, arg1) {
            const ret = getObject(arg0).push(getObject(arg1));
            return ret;
        };
        imports.wbg.__wbg_instanceof_ArrayBuffer_a69f02ee4c4f5065 = function(arg0) {
            let result;
            try {
                result = getObject(arg0) instanceof ArrayBuffer;
            } catch {
                result = false;
            }
            const ret = result;
            return ret;
        };
        imports.wbg.__wbg_call_9495de66fdbe016b = function() { return handleError(function (arg0, arg1, arg2) {
            const ret = getObject(arg0).call(getObject(arg1), getObject(arg2));
            return addHeapObject(ret);
        }, arguments) };
        imports.wbg.__wbg_call_96878afb7a8201ca = function() { return handleError(function (arg0, arg1, arg2, arg3) {
            const ret = getObject(arg0).call(getObject(arg1), getObject(arg2), getObject(arg3));
            return addHeapObject(ret);
        }, arguments) };
        imports.wbg.__wbg_set_388c4c6422704173 = function(arg0, arg1, arg2) {
            const ret = getObject(arg0).set(getObject(arg1), getObject(arg2));
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_isSafeInteger_8c4789029e885159 = function(arg0) {
            const ret = Number.isSafeInteger(getObject(arg0));
            return ret;
        };
        imports.wbg.__wbg_entries_4e1315b774245952 = function(arg0) {
            const ret = Object.entries(getObject(arg0));
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_buffer_cf65c07de34b9a08 = function(arg0) {
            const ret = getObject(arg0).buffer;
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_new_537b7341ce90bb31 = function(arg0) {
            const ret = new Uint8Array(getObject(arg0));
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_set_17499e8aa4003ebd = function(arg0, arg1, arg2) {
            getObject(arg0).set(getObject(arg1), arg2 >>> 0);
        };
        imports.wbg.__wbg_length_27a2afe8ab42b09f = function(arg0) {
            const ret = getObject(arg0).length;
            return ret;
        };
        imports.wbg.__wbg_instanceof_Uint8Array_01cebe79ca606cca = function(arg0) {
            let result;
            try {
                result = getObject(arg0) instanceof Uint8Array;
            } catch {
                result = false;
            }
            const ret = result;
            return ret;
        };
        imports.wbg.__wbg_newwithlength_b56c882b57805732 = function(arg0) {
            const ret = new Uint8Array(arg0 >>> 0);
            return addHeapObject(ret);
        };
        imports.wbg.__wbg_subarray_7526649b91a252a6 = function(arg0, arg1, arg2) {
            const ret = getObject(arg0).subarray(arg1 >>> 0, arg2 >>> 0);
            return addHeapObject(ret);
        };
        imports.wbg.__wbindgen_bigint_get_as_i64 = function(arg0, arg1) {
            const v = getObject(arg1);
            const ret = typeof(v) === 'bigint' ? v : undefined;
            getBigInt64Memory0()[arg0 / 8 + 1] = isLikeNone(ret) ? BigInt(0) : ret;
            getInt32Memory0()[arg0 / 4 + 0] = !isLikeNone(ret);
        };
        imports.wbg.__wbindgen_debug_string = function(arg0, arg1) {
            const ret = debugString(getObject(arg1));
            const ptr0 = passStringToWasm0(ret, wasm.__wbindgen_malloc, wasm.__wbindgen_realloc);
            const len0 = WASM_VECTOR_LEN;
            getInt32Memory0()[arg0 / 4 + 1] = len0;
            getInt32Memory0()[arg0 / 4 + 0] = ptr0;
        };
        imports.wbg.__wbindgen_throw = function(arg0, arg1) {
            throw new Error(getStringFromWasm0(arg0, arg1));
        };
        imports.wbg.__wbindgen_memory = function() {
            const ret = wasm.memory;
            return addHeapObject(ret);
        };

        return imports;
    }

    function finalizeInit(instance, module) {
        wasm = instance.exports;
        init.__wbindgen_wasm_module = module;
        cachedBigInt64Memory0 = null;
        cachedFloat32Memory0 = null;
        cachedFloat64Memory0 = null;
        cachedInt32Memory0 = null;
        cachedUint8Memory0 = null;


        return wasm;
    }

    function initSync(module) {
        const imports = getImports();

        if (!(module instanceof WebAssembly.Module)) {
            module = new WebAssembly.Module(module);
        }

        const instance = new WebAssembly.Instance(module, imports);

        return finalizeInit(instance, module);
    }

    async function init(input) {
        if (typeof input === 'undefined') {
            input = new URL('egraph_wasm_bg.wasm', (typeof document === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __filename).href : (document.currentScript && document.currentScript.src || new URL('egraph.js', document.baseURI).href)));
        }
        const imports = getImports();

        if (typeof input === 'string' || (typeof Request === 'function' && input instanceof Request) || (typeof URL === 'function' && input instanceof URL)) {
            input = fetch(input);
        }

        const { instance, module } = await load(await input, imports);

        return finalizeInit(instance, module);
    }

    exports.ClassicalMds = ClassicalMds;
    exports.CollideForce = CollideForce;
    exports.Coordinates = Coordinates;
    exports.DiGraph = DiGraph;
    exports.Drawing = Drawing;
    exports.ForceAtlas2 = ForceAtlas2;
    exports.FruchtermanReingoldForce = FruchtermanReingoldForce;
    exports.FullSgd = FullSgd;
    exports.Graph = Graph;
    exports.GroupLinkForce = GroupLinkForce;
    exports.GroupManyBodyForce = GroupManyBodyForce;
    exports.GroupPositionForce = GroupPositionForce;
    exports.HyperbolicSpace = HyperbolicSpace;
    exports.KamadaKawai = KamadaKawai;
    exports.LinkForce = LinkForce;
    exports.ManyBodyForce = ManyBodyForce;
    exports.PivotMds = PivotMds;
    exports.PositionForce = PositionForce;
    exports.RadialForce = RadialForce;
    exports.Rng = Rng;
    exports.SgdScheduler = SgdScheduler;
    exports.Simulation = Simulation;
    exports.SparseSgd = SparseSgd;
    exports.SphericalSpace = SphericalSpace;
    exports.StressMajorization = StressMajorization;
    exports.coarsen = coarsen;
    exports.crossingNumber = crossingNumber;
    exports.fdeb = fdeb;
    exports.fm3 = fm3;
    exports.init = init;
    exports.initSync = initSync;
    exports.neighborhoodPreservation = neighborhoodPreservation;
    exports.stress = stress;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
